var path = require('path')
var fs = require('fs')
var _ = require('lodash')
var axios = require('axios')
var Metalsmith = require('metalsmith')

var logger = require('./logger')

var config
var projectPath
var templatePath

function revertUrl(filters, url) {
  return filters.map(filter => {
    return url.indexOf(filter) !== -1 ? url : filter
  })
}

function init(apis, cb) {
  var helperPath = path.resolve(templatePath, '../helper/index.js')
  var helper = fs.existsSync(helperPath) ? require(helperPath) : {}
  var pathList = apis.modules;
  var basePath = apis.basePath;
  var dest = path.join(config.output, config.name || 'demo');
  build({
    config: config,
    basePath: basePath
  }, 'init', dest, cb)
  for (var p in pathList) {
    let item = pathList[p];
    var data = {
      data: item,
      config: config,
      mList: apis.mList,
      _: _
    }
    var _dest = path.join(dest, p)
    build(data, 'cover', _dest, cb)
  }
  if (fs.existsSync(path.join(templatePath, 'common'))) {
    build({
      config: config,
      mList: apis.mList,
      _: _,
      $$: helper
    }, 'common', config.output, cb, true)
  }
}

function build(data, source, dest, cb, ignore) {
  var metalsmith = Metalsmith(templatePath)
    .use(renderTemplateFiles(data))
    .clean(false)
    .source(source)
    .destination(dest)

  if (ignore) {
    metalsmith.ignore(filePath => {
      filePath = filePath.replace(path.join(templatePath, source), '')
      filePath = path.join(dest, filePath)
      return fs.existsSync(filePath)
    })
  }

  return metalsmith.build((error, files) => {
    if (error) logger.fatal(error)
    var f = Object.keys(files)
      .filter(o => fs.existsSync(path.join(dest, o)))
      .map(o => path.join(dest, o))
    cb(error, f)
  })
}

function renderTemplateFiles(data) {
  return function (files) {
    Object.keys(files).forEach((fileName) => {
      var file = files[fileName]
      file.contents = _.template(file.contents, {
        interpolate: /\{\{(.+?)\}\}/g
      })(data)
    })
  }
}

module.exports = function (_projectPath, _templatePath, _config, cb) {
  config = _config
  projectPath = _projectPath

  templatePath = path.join(_templatePath, 'template')
  config.output = path.resolve(projectPath, config.output || 'swagger-api')

  config.host = config.host || 'https://www.easy-mock.com'

  var jsonURL = config.swaggerJSON
  if (jsonURL.indexOf('http') <= -1) {
    jsonURL = path.join(projectPath, jsonURL)
  }
  axios
    .get(jsonURL, {})
    .then((res) => {
      try {
        var data;
        if (res.data.substr(0, 1) !== '{') {
          data = JSON.parse(res.data.substr(res.data.indexOf('{')))
        }
        var fData = {};
        fData.basePath = data.basePath;
        var phs = data.paths;
        var modules = {};
        for (let p in phs) {
          let item = phs[p]
          for (let pp in item) {
            let mName = item[pp]['tags'][0];
            let pName = p.substr(p.lastIndexOf('/') + 1)
            if (!modules[mName]) {
              modules[mName] = [];
            }
            modules[mName].push({
              path: p,
              method: pp,
              name: pName
            })
          }
        }
        var mmArr = [];
        for (let n in modules) {
          mmArr.push({
            name: n,
            list: modules[n]
          })
        }
        fData.mList = mmArr;
        fData.modules = modules;
        if (fData) {
          init(fData, cb)
        }
      } catch (e) {
        logger.fatal(e)
      }
    })
    .catch(error => {
      if (error.response) {
        logger.fatal(error.response.data)
      } else {
        logger.fatal(error)
      }
    })
}
